@@ -2,7 +2,7 @@
 /*
  * Virt Viewer: A virtual machine console viewer
  *
- * Copyright (C) 2012 Red Hat, Inc.
+ * Copyright (C) 2012-2015 Red Hat, Inc.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -39,6 +39,9 @@
  *
  *  The current list of [virt-viewer] keys is:
  * - version: string
+ * - versions: list of id:versions strings
+ * - newer-version-url: string specifying an URL to display when the minimum
+ *   version check fails
  * - type: string, mandatory, values: "spice" (later "vnc" etc..)
  * - host: string
  * - port: int
@@ -47,7 +50,7 @@
  * - password: string
  * - disable-channels: string list
  * - tls-ciphers: string
- * - ca: string PEM data (use \n to seperate the lines)
+ * - ca: string PEM data (use \n to separate the lines)
  * - host-subject: string
  * - fullscreen: int (0 or 1 atm)
  * - title: string
@@ -66,6 +69,19 @@
  * - delete-this-file: int (0 or 1 atm)
  * - proxy: proxy URL, like http://user:pass@foobar:8080
  *
+ * There is an optional [ovirt] section which can be used to specify
+ * the connection parameters to interact with the remote oVirt REST API.
+ * This is currently used to present a list of CDRom images which can be
+ * inserted in the VM.
+ *
+ * - host: string containing the URL of the oVirt engine
+ * - vm-guid: string containing the guid of the oVirt VM we are connecting to
+ * - jsessionid: string containing an authentication cookie to be used to
+ *   connect to the oVirt engine without being asked for credentials
+ * - ca: string PEM data (use \n to separate the lines)
+ * - admin: boolean (0 or 1) indicating whether the VM is visible in the user or
+ *   admin portal
+ *
  * (the file can be extended with extra groups or keys, which should
  * be prefixed with x- to avoid later conflicts)
  */
@@ -78,7 +94,8 @@ G_DEFINE_TYPE(VirtViewerFile, virt_viewer_file, G_TYPE_OBJECT);
 
 #define VIRT_VIEWER_FILE_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE((o), VIRT_VIEWER_TYPE_FILE, VirtViewerFilePrivate))
 
-#define GROUP "virt-viewer"
+#define MAIN_GROUP "virt-viewer"
+#define OVIRT_GROUP "ovirt"
 
 enum  {
     PROP_DUMMY_PROPERTY,
@@ -104,9 +121,16 @@ enum  {
     PROP_USB_FILTER,
     PROP_PROXY,
     PROP_VERSION,
+    PROP_VERSIONS,
+    PROP_VERSION_URL,
     PROP_SECURE_CHANNELS,
     PROP_DELETE_THIS_FILE,
     PROP_SECURE_ATTENTION,
+    PROP_OVIRT_ADMIN,
+    PROP_OVIRT_HOST,
+    PROP_OVIRT_VM_GUID,
+    PROP_OVIRT_JSESSIONID,
+    PROP_OVIRT_CA,
 };
 
 VirtViewerFile*
@@ -128,7 +152,7 @@ virt_viewer_file_new(const gchar* location, GError** error)
         return NULL;
     }
 
-    if (!g_key_file_has_group (keyfile, GROUP) ||
+    if (!g_key_file_has_group (keyfile, MAIN_GROUP) ||
         !virt_viewer_file_is_set(self, "type")) {
         inner_error = g_error_new_literal(G_KEY_FILE_ERROR,
                                           G_KEY_FILE_ERROR_NOT_FOUND, "Invalid file");
@@ -155,7 +179,7 @@ virt_viewer_file_is_set(VirtViewerFile* self, const gchar* key)
     g_return_val_if_fail(VIRT_VIEWER_IS_FILE(self), FALSE);
     g_return_val_if_fail(key != NULL, FALSE);
 
-    set = g_key_file_has_key(self->priv->keyfile, GROUP, key, &inner_error);
+    set = g_key_file_has_key(self->priv->keyfile, MAIN_GROUP, key, &inner_error);
     if (inner_error == NULL)
         return set;
     else {
@@ -165,17 +189,20 @@ virt_viewer_file_is_set(VirtViewerFile* self, const gchar* key)
 }
 
 static void
-virt_viewer_file_set_string(VirtViewerFile* self, const gchar* key, const gchar* value)
+virt_viewer_file_set_string(VirtViewerFile* self, const char *group,
+                            const gchar* key, const gchar* value)
 {
     g_return_if_fail(VIRT_VIEWER_IS_FILE(self));
     g_return_if_fail(key != NULL);
     g_return_if_fail(value != NULL);
 
-    g_key_file_set_string(self->priv->keyfile, GROUP, key, value);
+    g_key_file_set_string(self->priv->keyfile, group, key, value);
 }
 
 static gchar*
-virt_viewer_file_get_string(VirtViewerFile* self, const gchar* key)
+virt_viewer_file_get_string(VirtViewerFile* self,
+                            const char *group,
+                            const gchar* key)
 {
     GError* inner_error = NULL;
     gchar* result = NULL;
@@ -183,7 +210,7 @@ virt_viewer_file_get_string(VirtViewerFile* self, const gchar* key)
     g_return_val_if_fail(VIRT_VIEWER_IS_FILE(self), NULL);
     g_return_val_if_fail(key != NULL, NULL);
 
-    result = g_key_file_get_string(self->priv->keyfile, GROUP, key, &inner_error);
+    result = g_key_file_get_string(self->priv->keyfile, group, key, &inner_error);
     if (inner_error && inner_error->domain != G_KEY_FILE_ERROR)
         g_critical("%s", inner_error->message);
     g_clear_error(&inner_error);
@@ -192,16 +219,19 @@ virt_viewer_file_get_string(VirtViewerFile* self, const gchar* key)
 }
 
 static void
-virt_viewer_file_set_string_list(VirtViewerFile* self, const gchar* key, const gchar* const* value, gsize length)
+virt_viewer_file_set_string_list(VirtViewerFile* self, const char *group,
+                                 const gchar* key, const gchar* const* value,
+                                 gsize length)
 {
     g_return_if_fail(VIRT_VIEWER_IS_FILE(self));
     g_return_if_fail(key != NULL);
 
-    g_key_file_set_string_list(self->priv->keyfile, GROUP, key, value, length);
+    g_key_file_set_string_list(self->priv->keyfile, group, key, value, length);
 }
 
 static gchar**
-virt_viewer_file_get_string_list(VirtViewerFile* self, const gchar* key, gsize* length)
+virt_viewer_file_get_string_list(VirtViewerFile* self, const char *group,
+                                 const gchar* key, gsize* length)
 {
     GError* inner_error = NULL;
     gchar** result = NULL;
@@ -209,7 +239,7 @@ virt_viewer_file_get_string_list(VirtViewerFile* self, const gchar* key, gsize*
     g_return_val_if_fail(VIRT_VIEWER_IS_FILE(self), NULL);
     g_return_val_if_fail(key != NULL, NULL);
 
-    result = g_key_file_get_string_list(self->priv->keyfile, GROUP, key, length, &inner_error);
+    result = g_key_file_get_string_list(self->priv->keyfile, group, key, length, &inner_error);
     if (inner_error && inner_error->domain != G_KEY_FILE_ERROR)
         g_critical("%s", inner_error->message);
     g_clear_error(&inner_error);
@@ -218,16 +248,19 @@ virt_viewer_file_get_string_list(VirtViewerFile* self, const gchar* key, gsize*
 }
 
 static void
-virt_viewer_file_set_int(VirtViewerFile* self, const gchar* key, gint value)
+virt_viewer_file_set_int(VirtViewerFile* self, const char *group,
+                         const gchar* key, gint value)
 {
     g_return_if_fail(VIRT_VIEWER_IS_FILE(self));
     g_return_if_fail(key != NULL);
 
-    g_key_file_set_integer(self->priv->keyfile, GROUP, key, value);
+    g_key_file_set_integer(self->priv->keyfile, group, key, value);
 }
 
 static gint
-virt_viewer_file_get_int(VirtViewerFile* self, const gchar* key)
+virt_viewer_file_get_int(VirtViewerFile* self,
+                         const char *group,
+                         const gchar* key)
 {
     GError* inner_error = NULL;
     gint result;
@@ -235,7 +268,7 @@ virt_viewer_file_get_int(VirtViewerFile* self, const gchar* key)
     g_return_val_if_fail(VIRT_VIEWER_IS_FILE(self), -1);
     g_return_val_if_fail(key != NULL, -1);
 
-    result = g_key_file_get_integer(self->priv->keyfile, GROUP, key, &inner_error);
+    result = g_key_file_get_integer(self->priv->keyfile, group, key, &inner_error);
     if (inner_error && inner_error->domain != G_KEY_FILE_ERROR)
         g_critical("%s", inner_error->message);
     g_clear_error(&inner_error);
@@ -246,354 +279,498 @@ virt_viewer_file_get_int(VirtViewerFile* self, const gchar* key)
 gchar*
 virt_viewer_file_get_ca(VirtViewerFile* self)
 {
-    return virt_viewer_file_get_string(self, "ca");
+    return virt_viewer_file_get_string(self, MAIN_GROUP, "ca");
 }
 
 void
 virt_viewer_file_set_ca(VirtViewerFile* self, const gchar* value)
 {
-    virt_viewer_file_set_string(self, "ca", value);
+    virt_viewer_file_set_string(self, MAIN_GROUP, "ca", value);
     g_object_notify(G_OBJECT(self), "ca");
 }
 
 gchar*
 virt_viewer_file_get_host(VirtViewerFile* self)
 {
-    return virt_viewer_file_get_string(self, "host");
+    return virt_viewer_file_get_string(self, MAIN_GROUP, "host");
 }
 
 void
 virt_viewer_file_set_host(VirtViewerFile* self, const gchar* value)
 {
-    virt_viewer_file_set_string(self, "host", value);
+    virt_viewer_file_set_string(self, MAIN_GROUP, "host", value);
     g_object_notify(G_OBJECT(self), "host");
 }
 
 gchar*
 virt_viewer_file_get_file_type(VirtViewerFile* self)
 {
-    return virt_viewer_file_get_string(self, "type");
+    return virt_viewer_file_get_string(self, MAIN_GROUP, "type");
 }
 
 void
 virt_viewer_file_set_type(VirtViewerFile* self, const gchar* value)
 {
-    virt_viewer_file_set_string(self, "type", value);
+    virt_viewer_file_set_string(self, MAIN_GROUP, "type", value);
     g_object_notify(G_OBJECT(self), "type");
 }
 
 gint
 virt_viewer_file_get_port(VirtViewerFile* self)
 {
-    return virt_viewer_file_get_int(self, "port");
+    return virt_viewer_file_get_int(self, MAIN_GROUP, "port");
 }
 
 void
 virt_viewer_file_set_port(VirtViewerFile* self, gint value)
 {
-    virt_viewer_file_set_int(self, "port", value);
+    virt_viewer_file_set_int(self, MAIN_GROUP, "port", value);
     g_object_notify(G_OBJECT(self), "port");
 }
 
 gint
 virt_viewer_file_get_tls_port(VirtViewerFile* self)
 {
-    return virt_viewer_file_get_int(self, "tls-port");
+    return virt_viewer_file_get_int(self, MAIN_GROUP, "tls-port");
 }
 
 void
 virt_viewer_file_set_tls_port(VirtViewerFile* self, gint value)
 {
-    virt_viewer_file_set_int(self, "tls-port", value);
+    virt_viewer_file_set_int(self, MAIN_GROUP, "tls-port", value);
     g_object_notify(G_OBJECT(self), "tls-port");
 }
 
 gchar*
 virt_viewer_file_get_username(VirtViewerFile* self)
 {
-    return virt_viewer_file_get_string(self, "username");
+    return virt_viewer_file_get_string(self, MAIN_GROUP, "username");
 }
 
 gchar*
 virt_viewer_file_get_password(VirtViewerFile* self)
 {
-    return virt_viewer_file_get_string(self, "password");
+    return virt_viewer_file_get_string(self, MAIN_GROUP, "password");
 }
 
 void
 virt_viewer_file_set_username(VirtViewerFile* self, const gchar* value)
 {
-    virt_viewer_file_set_string(self, "username", value);
+    virt_viewer_file_set_string(self, MAIN_GROUP, "username", value);
     g_object_notify(G_OBJECT(self), "username");
 }
 
 void
 virt_viewer_file_set_password(VirtViewerFile* self, const gchar* value)
 {
-    virt_viewer_file_set_string(self, "password", value);
+    virt_viewer_file_set_string(self, MAIN_GROUP, "password", value);
     g_object_notify(G_OBJECT(self), "password");
 }
 
 gchar**
 virt_viewer_file_get_disable_channels(VirtViewerFile* self, gsize* length)
 {
-    return virt_viewer_file_get_string_list(self, "disable-channels", length);
+    return virt_viewer_file_get_string_list(self, MAIN_GROUP,
+                                            "disable-channels", length);
 }
 
 void
 virt_viewer_file_set_disable_channels(VirtViewerFile* self, const gchar* const* value, gsize length)
 {
-    virt_viewer_file_set_string_list(self, "disable-channels", value, length);
+    virt_viewer_file_set_string_list(self, MAIN_GROUP,
+                                     "disable-channels", value, length);
     g_object_notify(G_OBJECT(self), "disable-channels");
 }
 
 gchar**
 virt_viewer_file_get_disable_effects(VirtViewerFile* self, gsize* length)
 {
-    return virt_viewer_file_get_string_list(self, "disable-effects", length);
+    return virt_viewer_file_get_string_list(self, MAIN_GROUP,
+                                            "disable-effects", length);
 }
 
 void
 virt_viewer_file_set_disable_effects(VirtViewerFile* self, const gchar* const* value, gsize length)
 {
-    virt_viewer_file_set_string_list(self, "disable-effects", value, length);
+    virt_viewer_file_set_string_list(self, MAIN_GROUP,
+                                     "disable-effects", value, length);
     g_object_notify(G_OBJECT(self), "disable-effects");
 }
 
 gchar*
 virt_viewer_file_get_tls_ciphers(VirtViewerFile* self)
 {
-    return virt_viewer_file_get_string(self, "tls-ciphers");
+    return virt_viewer_file_get_string(self, MAIN_GROUP, "tls-ciphers");
 }
 
 void
 virt_viewer_file_set_tls_ciphers(VirtViewerFile* self, const gchar* value)
 {
-    virt_viewer_file_set_string(self, "tls-ciphers", value);
+    virt_viewer_file_set_string(self, MAIN_GROUP, "tls-ciphers", value);
     g_object_notify(G_OBJECT(self), "tls-ciphers");
 }
 
 gchar*
 virt_viewer_file_get_host_subject(VirtViewerFile* self)
 {
-    return virt_viewer_file_get_string(self, "host-subject");
+    return virt_viewer_file_get_string(self, MAIN_GROUP,
+                                       "host-subject");
 }
 
 void
 virt_viewer_file_set_host_subject(VirtViewerFile* self, const gchar* value)
 {
-    virt_viewer_file_set_string(self, "host-subject", value);
+    virt_viewer_file_set_string(self, MAIN_GROUP,
+                                "host-subject", value);
     g_object_notify(G_OBJECT(self), "host-subject");
 }
 
 gint
 virt_viewer_file_get_fullscreen(VirtViewerFile* self)
 {
-    return virt_viewer_file_get_int(self, "fullscreen");
+    return virt_viewer_file_get_int(self, MAIN_GROUP, "fullscreen");
 }
 
 void
 virt_viewer_file_set_fullscreen(VirtViewerFile* self, gint value)
 {
-    virt_viewer_file_set_int(self, "fullscreen", !!value);
+    virt_viewer_file_set_int(self, MAIN_GROUP, "fullscreen", !!value);
     g_object_notify(G_OBJECT(self), "fullscreen");
 }
 
 gchar*
 virt_viewer_file_get_title(VirtViewerFile* self)
 {
-    return virt_viewer_file_get_string(self, "title");
+    return virt_viewer_file_get_string(self, MAIN_GROUP, "title");
 }
 
 void
 virt_viewer_file_set_title(VirtViewerFile* self, const gchar* value)
 {
-    virt_viewer_file_set_string(self, "title", value);
+    virt_viewer_file_set_string(self, MAIN_GROUP, "title", value);
     g_object_notify(G_OBJECT(self), "title");
 }
 
 gchar*
 virt_viewer_file_get_toggle_fullscreen(VirtViewerFile* self)
 {
-    return virt_viewer_file_get_string(self, "toggle-fullscreen");
+    return virt_viewer_file_get_string(self, MAIN_GROUP, "toggle-fullscreen");
 }
 
 void
 virt_viewer_file_set_toggle_fullscreen(VirtViewerFile* self, const gchar* value)
 {
-    virt_viewer_file_set_string(self, "toggle-fullscreen", value);
+    virt_viewer_file_set_string(self, MAIN_GROUP, "toggle-fullscreen", value);
     g_object_notify(G_OBJECT(self), "toggle-fullscreen");
 }
 
 gchar*
 virt_viewer_file_get_release_cursor(VirtViewerFile* self)
 {
-    return virt_viewer_file_get_string(self, "release-cursor");
+    return virt_viewer_file_get_string(self, MAIN_GROUP, "release-cursor");
 }
 
 void
 virt_viewer_file_set_release_cursor(VirtViewerFile* self, const gchar* value)
 {
-    virt_viewer_file_set_string(self, "release-cursor", value);
+    virt_viewer_file_set_string(self, MAIN_GROUP, "release-cursor", value);
     g_object_notify(G_OBJECT(self), "release-cursor");
 }
 
 gchar*
 virt_viewer_file_get_secure_attention(VirtViewerFile* self)
 {
-    return virt_viewer_file_get_string(self, "secure-attention");
+    return virt_viewer_file_get_string(self, MAIN_GROUP, "secure-attention");
 }
 
 void
 virt_viewer_file_set_secure_attention(VirtViewerFile* self, const gchar* value)
 {
-    virt_viewer_file_set_string(self, "secure-attention", value);
+    virt_viewer_file_set_string(self, MAIN_GROUP, "secure-attention", value);
     g_object_notify(G_OBJECT(self), "secure-attention");
 }
 
 gchar*
 virt_viewer_file_get_smartcard_remove(VirtViewerFile* self)
 {
-    return virt_viewer_file_get_string(self, "smartcard-remove");
+    return virt_viewer_file_get_string(self, MAIN_GROUP, "smartcard-remove");
 }
 
 void
 virt_viewer_file_set_smartcard_remove(VirtViewerFile* self, const gchar* value)
 {
-    virt_viewer_file_set_string(self, "smartcard-remove", value);
+    virt_viewer_file_set_string(self, MAIN_GROUP, "smartcard-remove", value);
     g_object_notify(G_OBJECT(self), "smartcard-remove");
 }
 
 gchar*
 virt_viewer_file_get_smartcard_insert(VirtViewerFile* self)
 {
-    return virt_viewer_file_get_string(self, "smartcard-insert");
+    return virt_viewer_file_get_string(self, MAIN_GROUP, "smartcard-insert");
 }
 
 void
 virt_viewer_file_set_smartcard_insert(VirtViewerFile* self, const gchar* value)
 {
-    virt_viewer_file_set_string(self, "smartcard-insert", value);
+    virt_viewer_file_set_string(self, MAIN_GROUP, "smartcard-insert", value);
     g_object_notify(G_OBJECT(self), "smartcard-insert");
 }
 
 gint
 virt_viewer_file_get_enable_smartcard(VirtViewerFile* self)
 {
-    return virt_viewer_file_get_int(self, "enable-smartcard");
+    return virt_viewer_file_get_int(self, MAIN_GROUP, "enable-smartcard");
 }
 
 void
 virt_viewer_file_set_enable_smartcard(VirtViewerFile* self, gint value)
 {
-    virt_viewer_file_set_int(self, "enable-smartcard", !!value);
+    virt_viewer_file_set_int(self, MAIN_GROUP, "enable-smartcard", !!value);
     g_object_notify(G_OBJECT(self), "enable-smartcard");
 }
 
 gint
 virt_viewer_file_get_enable_usbredir(VirtViewerFile* self)
 {
-    return virt_viewer_file_get_int(self, "enable-usbredir");
+    return virt_viewer_file_get_int(self, MAIN_GROUP, "enable-usbredir");
 }
 
 void
 virt_viewer_file_set_enable_usbredir(VirtViewerFile* self, gint value)
 {
-    virt_viewer_file_set_int(self, "enable-usbredir", !!value);
+    virt_viewer_file_set_int(self, MAIN_GROUP, "enable-usbredir", !!value);
     g_object_notify(G_OBJECT(self), "enable-usbredir");
 }
 
 gint
 virt_viewer_file_get_delete_this_file(VirtViewerFile* self)
 {
-    return virt_viewer_file_get_int(self, "delete-this-file");
+    return virt_viewer_file_get_int(self, MAIN_GROUP, "delete-this-file");
 }
 
 void
 virt_viewer_file_set_delete_this_file(VirtViewerFile* self, gint value)
 {
-    virt_viewer_file_set_int(self, "delete-this-file", !!value);
+    virt_viewer_file_set_int(self, MAIN_GROUP, "delete-this-file", !!value);
     g_object_notify(G_OBJECT(self), "delete-this-file");
 }
 
 gint
 virt_viewer_file_get_color_depth(VirtViewerFile* self)
 {
-    return virt_viewer_file_get_int(self, "color-depth");
+    return virt_viewer_file_get_int(self, MAIN_GROUP, "color-depth");
 }
 
 void
 virt_viewer_file_set_color_depth(VirtViewerFile* self, gint value)
 {
-    virt_viewer_file_set_int(self, "color-depth", value);
+    virt_viewer_file_set_int(self, MAIN_GROUP, "color-depth", value);
     g_object_notify(G_OBJECT(self), "color-depth");
 }
 
 gint
 virt_viewer_file_get_enable_usb_autoshare(VirtViewerFile* self)
 {
-    return virt_viewer_file_get_int(self, "enable-usb-autoshare");
+    return virt_viewer_file_get_int(self, MAIN_GROUP, "enable-usb-autoshare");
 }
 
 void
 virt_viewer_file_set_enable_usb_autoshare(VirtViewerFile* self, gint value)
 {
-    virt_viewer_file_set_int(self, "enable-usb-autoshare", !!value);
+    virt_viewer_file_set_int(self, MAIN_GROUP, "enable-usb-autoshare", !!value);
     g_object_notify(G_OBJECT(self), "enable-usb-autoshare");
 }
 
 gchar*
 virt_viewer_file_get_usb_filter(VirtViewerFile* self)
 {
-    return virt_viewer_file_get_string(self, "usb-filter");
+    return virt_viewer_file_get_string(self, MAIN_GROUP, "usb-filter");
 }
 
 void
 virt_viewer_file_set_usb_filter(VirtViewerFile* self, const gchar* value)
 {
-    virt_viewer_file_set_string(self, "usb-filter", value);
+    virt_viewer_file_set_string(self, MAIN_GROUP, "usb-filter", value);
     g_object_notify(G_OBJECT(self), "usb-filter");
 }
 
 gchar*
 virt_viewer_file_get_proxy(VirtViewerFile* self)
 {
-    return virt_viewer_file_get_string(self, "proxy");
+    return virt_viewer_file_get_string(self, MAIN_GROUP, "proxy");
 }
 
 void
 virt_viewer_file_set_proxy(VirtViewerFile* self, const gchar* value)
 {
-    virt_viewer_file_set_string(self, "proxy", value);
+    virt_viewer_file_set_string(self, MAIN_GROUP, "proxy", value);
     g_object_notify(G_OBJECT(self), "proxy");
 }
 
 gchar*
 virt_viewer_file_get_version(VirtViewerFile* self)
 {
-    return virt_viewer_file_get_string(self, "version");
+    return virt_viewer_file_get_string(self, MAIN_GROUP, "version");
 }
 
 void
 virt_viewer_file_set_version(VirtViewerFile* self, const gchar* value)
 {
-    virt_viewer_file_set_string(self, "version", value);
+    virt_viewer_file_set_string(self, MAIN_GROUP, "version", value);
     g_object_notify(G_OBJECT(self), "version");
 }
 
+GHashTable*
+virt_viewer_file_get_versions(VirtViewerFile* self)
+{
+    GHashTable *versions;
+    gchar **versions_str;
+    gsize length;
+    unsigned int i;
+
+    versions = g_hash_table_new_full(g_str_hash, g_str_equal, g_free, g_free);
+    versions_str = virt_viewer_file_get_string_list(self, MAIN_GROUP,
+                                                     "versions", &length);
+    for (i = 0; i < length; i++) {
+        GStrv tokens;
+
+        if (versions_str[i] == NULL) {
+            g_warn_if_reached();
+            break;
+        }
+        tokens = g_strsplit(versions_str[i], ":", 2);
+        if (g_strv_length(tokens) != 2) {
+            g_warn_if_reached();
+            continue;
+        }
+        g_debug("Minimum version '%s' for OS id '%s'", tokens[1], tokens[0]);
+        g_hash_table_insert(versions, tokens[0], tokens[1]);
+        g_free(tokens);
+    }
+    g_strfreev(versions_str);
+
+    return versions;
+}
+
+void
+virt_viewer_file_set_versions(VirtViewerFile* self, GHashTable *version_table)
+{
+    GHashTableIter iter;
+    gpointer key, value;
+    GPtrArray *versions;
+
+    versions = g_ptr_array_new_with_free_func(g_free);
+
+    g_hash_table_iter_init(&iter, version_table);
+    while (g_hash_table_iter_next(&iter, &key, &value)) {
+        char *str;
+
+        /* Check that id only contains letters/numbers/- */
+        /* Check that version only contains numbers, ., :, -, (letters ?) */
+        /* FIXME: ':' separator overlaps with ':' epoch indicator */
+
+        str = g_strdup_printf("%s:%s", (char *)key, (char *)value);
+        g_ptr_array_add(versions, str);
+    }
+    virt_viewer_file_set_string_list(self, MAIN_GROUP, "versions",
+                                     (const char * const *)versions->pdata,
+                                     versions->len);
+    g_ptr_array_unref(versions);
+    g_object_notify(G_OBJECT(self), "versions");
+}
+
+gchar*
+virt_viewer_file_get_version_url(VirtViewerFile* self)
+{
+    return virt_viewer_file_get_string(self, MAIN_GROUP, "newer-version-url");
+}
+
+void
+virt_viewer_file_set_version_url(VirtViewerFile* self, const gchar* value)
+{
+    virt_viewer_file_set_string(self, MAIN_GROUP, "newer-version-url", value);
+    g_object_notify(G_OBJECT(self), "version-url");
+}
+
+
 gchar**
 virt_viewer_file_get_secure_channels(VirtViewerFile* self, gsize* length)
 {
-    return virt_viewer_file_get_string_list(self, "secure-channels", length);
+    return virt_viewer_file_get_string_list(self, MAIN_GROUP, "secure-channels", length);
 }
 
 void
 virt_viewer_file_set_secure_channels(VirtViewerFile* self, const gchar* const* value, gsize length)
 {
-    virt_viewer_file_set_string_list(self, "secure-channels", value, length);
+    virt_viewer_file_set_string_list(self, MAIN_GROUP, "secure-channels", value, length);
     g_object_notify(G_OBJECT(self), "secure-channels");
 }
 
+gchar*
+virt_viewer_file_get_ovirt_host(VirtViewerFile* self)
+{
+    return virt_viewer_file_get_string(self, OVIRT_GROUP, "host");
+}
+
+void
+virt_viewer_file_set_ovirt_host(VirtViewerFile* self, const gchar* value)
+{
+    virt_viewer_file_set_string(self, OVIRT_GROUP, "host", value);
+    g_object_notify(G_OBJECT(self), "ovirt-host");
+}
+
+gchar*
+virt_viewer_file_get_ovirt_vm_guid(VirtViewerFile* self)
+{
+    return virt_viewer_file_get_string(self, OVIRT_GROUP, "vm-guid");
+}
+
+void
+virt_viewer_file_set_ovirt_vm_guid(VirtViewerFile* self, const gchar* value)
+{
+    virt_viewer_file_set_string(self, OVIRT_GROUP, "vm-guid", value);
+    g_object_notify(G_OBJECT(self), "ovirt-vm-guid");
+}
+
+gchar*
+virt_viewer_file_get_ovirt_jsessionid(VirtViewerFile* self)
+{
+    return virt_viewer_file_get_string(self, OVIRT_GROUP, "jsessionid");
+}
+
+void
+virt_viewer_file_set_ovirt_jsessionid(VirtViewerFile* self, const gchar* value)
+{
+    virt_viewer_file_set_string(self, OVIRT_GROUP, "jsessionid", value);
+    g_object_notify(G_OBJECT(self), "ovirt-jsessionid");
+}
+
+gchar*
+virt_viewer_file_get_ovirt_ca(VirtViewerFile* self)
+{
+    return virt_viewer_file_get_string(self, OVIRT_GROUP, "ca");
+}
+
+void
+virt_viewer_file_set_ovirt_ca(VirtViewerFile* self, const gchar* value)
+{
+    virt_viewer_file_set_string(self, OVIRT_GROUP, "ca", value);
+    g_object_notify(G_OBJECT(self), "ovirt-ca");
+}
+
+gint
+virt_viewer_file_get_ovirt_admin(VirtViewerFile* self)
+{
+    return virt_viewer_file_get_int(self, OVIRT_GROUP, "admin");
+}
+
+void
+virt_viewer_file_set_ovirt_admin(VirtViewerFile* self, gint value)
+{
+    virt_viewer_file_set_int(self, OVIRT_GROUP, "admin", value);
+    g_object_notify(G_OBJECT(self), "ovirt-admin");
+}
+
 static void
 spice_hotkey_set_accel(const gchar *accel_path, const gchar *key)
 {
@@ -608,27 +785,70 @@ spice_hotkey_set_accel(const gchar *accel_path, const gchar *key)
     gtk_accel_map_change_entry(accel_path, accel_key, accel_mods, TRUE);
 }
 
+static gboolean
+virt_viewer_file_check_min_version(VirtViewerFile *self, GError **error)
+{
+    gchar *min_version = NULL;
+    gint version_cmp;
+
+#ifdef REMOTE_VIEWER_OS_ID
+    if (virt_viewer_file_is_set(self, "versions")) {
+        GHashTable *versions;
+
+        versions = virt_viewer_file_get_versions(self);
+
+        min_version = g_strdup(g_hash_table_lookup(versions, REMOTE_VIEWER_OS_ID));
+
+        g_hash_table_unref(versions);
+    }
+#endif
+
+
+    if (min_version == NULL) {
+        if (virt_viewer_file_is_set(self, "version")) {
+            min_version = virt_viewer_file_get_version(self);
+        }
+    }
+
+    if (min_version == NULL) {
+        return TRUE;
+    }
+    version_cmp = virt_viewer_compare_buildid(min_version, PACKAGE_VERSION BUILDID);
+
+    if (version_cmp > 0) {
+        gchar *url;
+        url = virt_viewer_file_get_version_url(self);
+        if (url != NULL) {
+            g_set_error(error,
+                        VIRT_VIEWER_ERROR,
+                        VIRT_VIEWER_ERROR_FAILED,
+                        _("At least %s version %s is required to setup this"
+                          " connection, see %s for details"),
+                        g_get_application_name(), min_version, url);
+            g_free(url);
+        } else {
+            g_set_error(error,
+                        VIRT_VIEWER_ERROR,
+                        VIRT_VIEWER_ERROR_FAILED,
+                        _("At least %s version %s is required to setup this connection"),
+                        g_get_application_name(), min_version);
+        }
+        g_free(min_version);
+        return FALSE;
+    }
+    g_free(min_version);
+
+    return TRUE;
+}
+
 gboolean
 virt_viewer_file_fill_app(VirtViewerFile* self, VirtViewerApp *app, GError **error)
 {
     g_return_val_if_fail(VIRT_VIEWER_IS_FILE(self), FALSE);
     g_return_val_if_fail(VIRT_VIEWER_IS_APP(app), FALSE);
 
-    if (virt_viewer_file_is_set(self, "version")) {
-        gchar *val = virt_viewer_file_get_version(self);
-
-        if (virt_viewer_compare_version(val, PACKAGE_VERSION) > 0) {
-            g_set_error(error,
-                VIRT_VIEWER_ERROR,
-                VIRT_VIEWER_ERROR_FAILED,
-                _("At least %s version %s is required to setup this connection"),
-                g_get_application_name(), val);
-
-            g_free(val);
-            return FALSE;
-        }
-
-        g_free(val);
+    if (!virt_viewer_file_check_min_version(self, error)) {
+        return FALSE;
     }
 
     if (virt_viewer_file_is_set(self, "title")) {
@@ -750,6 +970,12 @@ virt_viewer_file_set_property(GObject* object, guint property_id,
     case PROP_VERSION:
         virt_viewer_file_set_version(self, g_value_get_string(value));
         break;
+    case PROP_VERSIONS:
+        virt_viewer_file_set_versions(self, g_value_get_boxed(value));
+        break;
+    case PROP_VERSION_URL:
+        virt_viewer_file_set_version_url(self, g_value_get_string(value));
+        break;
     case PROP_SECURE_CHANNELS:
         strv = g_value_get_boxed(value);
         virt_viewer_file_set_secure_channels(self, (const gchar* const*)strv, g_strv_length(strv));
@@ -757,6 +983,21 @@ virt_viewer_file_set_property(GObject* object, guint property_id,
     case PROP_DELETE_THIS_FILE:
         virt_viewer_file_set_delete_this_file(self, g_value_get_int(value));
         break;
+    case PROP_OVIRT_ADMIN:
+        virt_viewer_file_set_ovirt_admin(self, g_value_get_int(value));
+        break;
+    case PROP_OVIRT_HOST:
+        virt_viewer_file_set_ovirt_host(self, g_value_get_string(value));
+        break;
+    case PROP_OVIRT_VM_GUID:
+        virt_viewer_file_set_ovirt_vm_guid(self, g_value_get_string(value));
+        break;
+    case PROP_OVIRT_JSESSIONID:
+        virt_viewer_file_set_ovirt_jsessionid(self, g_value_get_string(value));
+        break;
+    case PROP_OVIRT_CA:
+        virt_viewer_file_set_ovirt_ca(self, g_value_get_string(value));
+        break;
     default:
         G_OBJECT_WARN_INVALID_PROPERTY_ID(object, property_id, pspec);
         break;
@@ -839,12 +1080,33 @@ virt_viewer_file_get_property(GObject* object, guint property_id,
     case PROP_VERSION:
         g_value_take_string(value, virt_viewer_file_get_version(self));
         break;
+    case PROP_VERSIONS:
+        g_value_take_boxed(value, virt_viewer_file_get_versions(self));
+        break;
+    case PROP_VERSION_URL:
+        g_value_take_string(value, virt_viewer_file_get_version_url(self));
+        break;
     case PROP_SECURE_CHANNELS:
         g_value_take_boxed(value, virt_viewer_file_get_secure_channels(self, NULL));
         break;
     case PROP_DELETE_THIS_FILE:
         g_value_set_int(value, virt_viewer_file_get_delete_this_file(self));
         break;
+    case PROP_OVIRT_ADMIN:
+        g_value_set_int(value, virt_viewer_file_get_ovirt_admin(self));
+        break;
+    case PROP_OVIRT_HOST:
+        g_value_take_string(value, virt_viewer_file_get_ovirt_host(self));
+        break;
+    case PROP_OVIRT_VM_GUID:
+        g_value_take_string(value, virt_viewer_file_get_ovirt_vm_guid(self));
+        break;
+    case PROP_OVIRT_JSESSIONID:
+        g_value_take_string(value, virt_viewer_file_get_ovirt_jsessionid(self));
+        break;
+    case PROP_OVIRT_CA:
+        g_value_take_string(value, virt_viewer_file_get_ovirt_ca(self));
+        break;
     default:
         G_OBJECT_WARN_INVALID_PROPERTY_ID(object, property_id, pspec);
         break;
@@ -972,6 +1234,14 @@ virt_viewer_file_class_init(VirtViewerFileClass* klass)
         g_param_spec_string("version", "version", "version", NULL,
                             G_PARAM_STATIC_STRINGS | G_PARAM_READWRITE));
 
+    g_object_class_install_property(G_OBJECT_CLASS(klass), PROP_VERSIONS,
+        g_param_spec_boxed("versions", "versions", "versions", G_TYPE_HASH_TABLE,
+                           G_PARAM_STATIC_STRINGS | G_PARAM_READWRITE));
+
+    g_object_class_install_property(G_OBJECT_CLASS(klass), PROP_VERSION_URL,
+        g_param_spec_string("version-url", "version-url", "version-url", NULL,
+                            G_PARAM_STATIC_STRINGS | G_PARAM_READWRITE));
+
     g_object_class_install_property(G_OBJECT_CLASS(klass), PROP_SECURE_CHANNELS,
         g_param_spec_boxed("secure-channels", "secure-channels", "secure-channels", G_TYPE_STRV,
                            G_PARAM_STATIC_STRINGS | G_PARAM_READWRITE));
@@ -979,4 +1249,24 @@ virt_viewer_file_class_init(VirtViewerFileClass* klass)
     g_object_class_install_property(G_OBJECT_CLASS(klass), PROP_DELETE_THIS_FILE,
         g_param_spec_int("delete-this-file", "delete-this-file", "delete-this-file", 0, 1, 0,
                          G_PARAM_STATIC_STRINGS | G_PARAM_READWRITE));
+
+    g_object_class_install_property(G_OBJECT_CLASS(klass), PROP_OVIRT_ADMIN,
+        g_param_spec_int("ovirt-admin", "ovirt-admin", "ovirt-admin", 0, 1, 0,
+                         G_PARAM_STATIC_STRINGS | G_PARAM_READWRITE));
+
+    g_object_class_install_property(G_OBJECT_CLASS(klass), PROP_OVIRT_HOST,
+        g_param_spec_string("ovirt-host", "ovirt-host", "ovirt-host", NULL,
+                            G_PARAM_STATIC_STRINGS | G_PARAM_READWRITE));
+
+    g_object_class_install_property(G_OBJECT_CLASS(klass), PROP_OVIRT_VM_GUID,
+        g_param_spec_string("ovirt-vm-guid", "ovirt-vm-guid", "ovirt-vm-guid", NULL,
+                            G_PARAM_STATIC_STRINGS | G_PARAM_READWRITE));
+
+    g_object_class_install_property(G_OBJECT_CLASS(klass), PROP_OVIRT_JSESSIONID,
+        g_param_spec_string("ovirt-jsessionid", "ovirt-jsessionid", "ovirt-jsessionid", NULL,
+                            G_PARAM_STATIC_STRINGS | G_PARAM_READWRITE));
+
+    g_object_class_install_property(G_OBJECT_CLASS(klass), PROP_OVIRT_CA,
+        g_param_spec_string("ovirt-ca", "ovirt-ca", "ovirt-ca", NULL,
+                            G_PARAM_STATIC_STRINGS | G_PARAM_READWRITE));
 }
